@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2010-2012 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2010-2013 Matthias Klumpp <matthias@tenstral.net>
  *                    2011 Richard Hughes <richard@hughsie.com>
  * Licensed under the GNU Lesser General Public License Version 3
  *
@@ -20,6 +20,7 @@
 
 #include <stdlib.h>
 #include <glib.h>
+#include <locale.h>
 #include <packagekit-glib2/packagekit.h>
 #include <plugin/packagekit-plugin.h>
 
@@ -29,16 +30,14 @@
 struct PkPluginPrivate {
 	PkTransaction		*current_transaction;
 	ListallerManager	*mgr;
-	ListallerSettings	*conf;
+	ListallerSetupSettings	*setup_settings;
 	PkResults		*backend_results;
 	GMainLoop		*loop;
+	gboolean		error_set;
 };
 
-typedef struct {
-	guint finished_id;
-	guint package_id;
-	guint error_code_id;
-} PkPluginSignalData;
+static void pk_plugin_reset_backend_job (PkPlugin *plugin);
+static void pk_plugin_redirect_backend_signals (PkPlugin *plugin);
 
 /**
  * pk_plugin_reset:
@@ -46,12 +45,12 @@ typedef struct {
 void
 pk_plugin_reset (PkPlugin *plugin)
 {
-	/* reset the native backend */
-	pk_backend_reset (plugin->backend);
-
 	/* recreate PkResults object */
 	g_object_unref (plugin->priv->backend_results);
 	plugin->priv->backend_results = pk_results_new ();
+
+	/* reset the job */
+	pk_plugin_reset_backend_job (plugin);
 }
 
 /**
@@ -63,34 +62,40 @@ static void
 pk_plugin_skip_native_backend (PkPlugin *plugin)
 {
 	PkExitEnum exit;
-	exit = pk_backend_get_exit_code (plugin->backend);
+	exit = pk_backend_job_get_exit_code (plugin->job);
 
 	/* only skip transaction if we don't have an error already */
-	if (!pk_backend_get_is_error_set (plugin->backend)) {
-		pk_backend_set_exit_code (plugin->backend, PK_EXIT_ENUM_SKIP_TRANSACTION);
+	if (!pk_backend_job_get_is_error_set (plugin->job)) {
+		pk_backend_job_set_exit_code (plugin->job, PK_EXIT_ENUM_SKIP_TRANSACTION);
 	}
 }
 
 /**
- * pk_plugin_listaller_scan_applications:
+ * pk_plugin_listaller_refresh_repos:
  *
- * Updates the current application database
+ * Updates the current application caches
  *
  * Return value: True if successful
  */
 gboolean
-pk_plugin_listaller_scan_applications (PkPlugin *plugin)
+pk_plugin_listaller_refresh_repos (PkPlugin *plugin)
 {
 	gboolean res;
+	ListallerManager *limgr;
+
+	/* create new Listaller task */
+	limgr = listaller_manager_new (TRUE);
 
 	/* run it */
-	res = listaller_manager_scan_applications (plugin->priv->mgr);
+	res = listaller_manager_refresh_repository_cache (limgr);
 
-	/* print warning if scan fails */
+	g_object_unref (limgr);
+
+	/* print warning if refresh fails */
 	if (!res) {
-		g_warning ("listaller: unable to update application database.");
+		g_warning ("listaller: unable to update application data from repositories.");
 	} else {
-		g_debug ("listaller: application database update finished.");
+		g_debug ("listaller: application cache update finished.");
 	}
 
 	return res;
@@ -106,7 +111,7 @@ pk_listaller_find_applications (PkPlugin *plugin, gchar **values)
 {
 	g_debug ("listaller: searching for applications: %s", values[0]);
 	listaller_manager_find_applications_by_values (plugin->priv->mgr,
-						       LISTALLER_APP_SOURCE_EXTERN,
+						       "*",
 						       values,
 						       NULL);
 }
@@ -147,13 +152,13 @@ pk_listaller_get_details (PkPlugin *plugin, gchar **package_ids)
 	guint i;
 
 	g_debug ("listaller: running get_details ()");
-	pk_backend_reset (plugin->backend);
+	pk_plugin_reset_backend_job (plugin);
 
 	for (i=0; package_ids[i] != NULL; i++) {
 		app = pk_listaller_appitem_from_pkid (package_ids[i]);
 
 		/* update AppItem with database data */
-		listaller_manager_refresh_appitem (plugin->priv->mgr, &app);
+		listaller_manager_refresh_appitem_data (plugin->priv->mgr, &app);
 
 		description = listaller_app_item_get_description (app);
 		listaller_app_item_get_license (app, &license);
@@ -162,7 +167,7 @@ pk_listaller_get_details (PkPlugin *plugin, gchar **package_ids)
 		//TODO: Fetch size of installed application too
 
 		/* emit */
-		pk_backend_details (plugin->backend, package_ids[i],
+		pk_backend_job_details (plugin->job, package_ids[i],
 					license.name,
 					PK_GROUP_ENUM_UNKNOWN,
 					description,
@@ -182,24 +187,25 @@ pk_listaller_get_filelist (PkPlugin *plugin, gchar **package_ids)
 	guint i;
 
 	g_debug ("listaller: running get_filelist ()");
-	pk_backend_reset (plugin->backend);
+	pk_plugin_reset_backend_job (plugin);
 
 	for (i=0; package_ids[i] != NULL; i++) {
 		app = pk_listaller_appitem_from_pkid (package_ids[i]);
 
 		/* update AppItem with database data */
-		listaller_manager_refresh_appitem (plugin->priv->mgr, &app);
+		listaller_manager_refresh_appitem_data (plugin->priv->mgr, &app);
 
 		filelist = listaller_manager_get_application_filelist_as_string (plugin->priv->mgr, app);
 		if (filelist == NULL)
 			filelist = "ERROR while fetching list of files. (Please report this issue)";
 
 		/* emit */
-		pk_backend_files (plugin->backend, package_ids[i], filelist);
+		pk_backend_job_files (plugin->job, package_ids[i], filelist);
 	};
 }
 
-static void listaller_application_cb (GObject *sender, ListallerAppItem *item, PkPlugin *plugin)
+static void
+listaller_application_cb (GObject *sender, ListallerAppItem *item, PkPlugin *plugin)
 {
 	gchar *package_id;
 
@@ -210,27 +216,39 @@ static void listaller_application_cb (GObject *sender, ListallerAppItem *item, P
 	}
 	g_debug ("listaller: new app found -> %s", listaller_app_item_get_appid (item));
 
+	pk_plugin_reset_backend_job (plugin);
+
 	/* emit */
-	pk_backend_package (plugin->backend, PK_INFO_ENUM_INSTALLED, package_id,
+	pk_backend_job_package (plugin->job, PK_INFO_ENUM_INSTALLED, package_id,
 			    listaller_app_item_get_summary (item));
 
+	pk_plugin_redirect_backend_signals (plugin);
+
 	g_free (package_id);
 }
 
-static void listaller_error_code_cb (GObject *sender, ListallerErrorItem *error, PkPlugin *plugin)
+static void
+listaller_error_code_cb (GObject *sender, ListallerErrorItem *error, PkPlugin *plugin)
 {
 	g_return_if_fail (error != NULL);
 
 	/* don't try to set errors twice */
-	if (pk_backend_get_is_error_set (plugin->backend))
+	if (pk_backend_job_get_is_error_set (plugin->job))
 		return;
 
+	/* set error code to prevent other actions to continue, after the transaction has failed */
+	plugin->priv->error_set = TRUE;
+
+	/* we want this to be sent to PackageKit clients */
+	pk_plugin_reset_backend_job (plugin);
+
 	/* emit */
-	pk_backend_error_code (plugin->backend, PK_ERROR_ENUM_INTERNAL_ERROR,
-				listaller_error_item_get_details (error));
+	pk_backend_job_error_code (plugin->job, PK_ERROR_ENUM_INTERNAL_ERROR,
+				"%s", listaller_error_item_get_details (error));
 }
 
-static void listaller_message_cb (GObject *sender, ListallerMessageItem *message, PkPlugin *plugin)
+static void
+listaller_message_cb (GObject *sender, ListallerMessageItem *message, PkPlugin *plugin)
 {
 	ListallerMessageEnum mtype;
 	gchar *text;
@@ -255,14 +273,34 @@ static void listaller_message_cb (GObject *sender, ListallerMessageItem *message
 }
 
 
-static void listaller_progress_change_cb (GObject *sender, gint progress, gint subprogress, PkPlugin *plugin)
+static void
+listaller_progress_cb (GObject *sender, ListallerProgressItem *item, PkPlugin *plugin)
 {
+	gint value;
+	ListallerProgressEnum prog_type;
+	value = listaller_progress_item_get_value (item);
+	prog_type = listaller_progress_item_get_prog_type (item);
+
+	pk_plugin_reset_backend_job (plugin);
+
 	/* emit */
-	pk_backend_set_percentage (plugin->backend, progress);
-	pk_backend_set_sub_percentage (plugin->backend, subprogress);
+	if (prog_type == LISTALLER_PROGRESS_ENUM_MAIN_PROGRESS) {
+		if (value > 0)
+			pk_backend_job_set_percentage (plugin->job, value);
+	}
+
+	if (prog_type == LISTALLER_PROGRESS_ENUM_ITEM_PROGRESS) {
+		const gchar *item_id = listaller_progress_item_get_item_id (item);
+		// TODO: Implement better status handling
+		if (value > 0)
+			pk_backend_job_set_item_progress (plugin->job, item_id, PK_STATUS_ENUM_RUNNING, value);
+	}
+
+	pk_plugin_redirect_backend_signals (plugin);
 }
 
-static void listaller_status_change_cb (GObject *sender, ListallerStatusItem *status, PkPlugin *plugin)
+static void
+listaller_status_change_cb (GObject *sender, ListallerStatusItem *status, PkPlugin *plugin)
 {
 	ListallerStatusEnum listatus;
 	PkStatusEnum pkstatus;
@@ -279,9 +317,13 @@ static void listaller_status_change_cb (GObject *sender, ListallerStatusItem *st
 
 	g_debug ("listaller: <status-info> %s", listaller_status_item_get_info (status));
 
+	pk_plugin_reset_backend_job (plugin);
+
 	/* emit */
 	if (pkstatus != PK_STATUS_ENUM_UNKNOWN)
-		pk_backend_set_status (plugin->backend, pkstatus);
+		pk_backend_job_set_status (plugin->job, pkstatus);
+
+	pk_plugin_redirect_backend_signals (plugin);
 }
 
 /**
@@ -299,15 +341,15 @@ pk_listaller_install_file (PkPlugin *plugin, const gchar *filename)
 	gchar* package_id;
 	ListallerAppItem *app = NULL;
 
-	setup = listaller_setup_new (filename, plugin->priv->conf);
+	setup = listaller_setup_new (filename);
 	g_signal_connect (setup, "error-code",
 			  G_CALLBACK (listaller_error_code_cb), plugin);
 	g_signal_connect (setup, "message",
 			  G_CALLBACK (listaller_message_cb), plugin);
 	g_signal_connect (setup, "status-changed",
 			  G_CALLBACK (listaller_status_change_cb), plugin);
-	g_signal_connect (setup, "progress-changed",
-			  G_CALLBACK (listaller_progress_change_cb), plugin);
+	g_signal_connect (setup, "progress",
+			  G_CALLBACK (listaller_progress_cb), plugin);
 
 	/* now intialize the new setup */
 	ret = listaller_setup_initialize (setup);
@@ -316,6 +358,8 @@ pk_listaller_install_file (PkPlugin *plugin, const gchar *filename)
 
 	/* install the application! */
 	ret = listaller_setup_run_installation (setup);
+	if (!ret)
+		goto out;
 
 	/* fetch installed application */
 	app = listaller_setup_get_current_application (setup);
@@ -323,19 +367,19 @@ pk_listaller_install_file (PkPlugin *plugin, const gchar *filename)
 	package_id = pk_listaller_pkid_from_appitem (app);
 	if (package_id == NULL) {
 		g_debug ("listaller: <error> Unable to build package-id from app-id!");
-	} else if (!pk_backend_get_is_error_set (plugin->backend)) {
+	} else if (!pk_backend_job_get_is_error_set (plugin->job)) {
 		/* emit */
-		pk_backend_package (plugin->backend, PK_INFO_ENUM_INSTALLED,
+		pk_backend_job_package (plugin->job, PK_INFO_ENUM_INSTALLED,
 					package_id,
 					listaller_app_item_get_summary (app));
 		g_free (package_id);
 	}
 	g_object_unref (app);
 
+out:
 	/* close setup */
 	g_object_unref (setup);
 
-out:
 	return ret;
 }
 
@@ -360,10 +404,108 @@ pk_listaller_install_files (PkPlugin *plugin, gchar **filenames)
 }
 
 /**
- * pk_plugin_backend_package_cb:
+ * pk_listaller_get_updates:
+ *
+ * Submit available Listaller software updates
+ *
+ * Return value: True if successful
+ **/
+static gboolean
+pk_listaller_get_updates (PkPlugin *plugin)
+{
+	gboolean ret = TRUE;
+	ListallerUpdater *upd;
+	GeeArrayList *available_updates;
+	ListallerUpdateItem *uitem;
+	ListallerAppItem *app;
+	ListallerIPKDependency *dep;
+	gchar *package_id;
+	GType swtype;
+	gpointer ptr;
+	guint i;
+
+	/* create new update manager and connect it */
+	upd = listaller_updater_new (TRUE);
+	g_signal_connect (upd, "error-code", G_CALLBACK (listaller_error_code_cb), plugin);
+	g_signal_connect (upd, "progress", G_CALLBACK (listaller_progress_cb), plugin);
+
+	/* find available updates */
+	listaller_updater_find_updates (upd);
+
+	/* fetch found updates & emit them to frontend */
+	available_updates = listaller_updater_get_available_updates (upd);
+
+	for (i = 0; i < gee_abstract_collection_get_size ((GeeAbstractCollection*) available_updates); i++) {
+		ptr = gee_abstract_list_get ((GeeAbstractList*) available_updates, i);
+		uitem = (ListallerUpdateItem*) ptr;
+
+		swtype = listaller_update_item_get_sw_type (uitem);
+		if (swtype == LISTALLER_TYPE_APP_ITEM) {
+			app = (ListallerAppItem*) listaller_update_item_get_sw_new (uitem);
+			package_id = pk_listaller_pkid_from_appitem (app);
+			if (package_id == NULL) {
+				g_warning ("listaller: Unable to build package-id from app-id!");
+				ret = FALSE;
+				continue;
+			}
+
+			g_debug ("Emitting update package: %s", package_id);
+			pk_backend_job_package (plugin->job, PK_INFO_ENUM_NORMAL,
+						package_id,
+						listaller_app_item_get_summary (app));
+			g_free (package_id);
+		} else if (swtype == LISTALLER_IPK_TYPE_DEPENDENCY) {
+			// TODO
+		} else {
+			g_warning ("Found UpdateItem with invalid software type: %s", g_type_name (swtype));
+		}
+
+	}
+
+out:
+	g_object_unref (upd);
+
+	return ret;
+}
+
+/**
+ * pk_listaller_apply_updates:
+ *
+ * Apply selected Listaller software updates
+ *
+ * Return value: True if successful
+ **/
+static gboolean
+pk_listaller_apply_updates (PkPlugin *plugin, gchar** package_ids)
+{
+	gboolean ret = TRUE;
+	ListallerUpdater *upd;
+	GeeArrayList *selected_updates;
+
+	/* create new update manager and connect it */
+	upd = listaller_updater_new (TRUE);
+	g_signal_connect (upd, "error-code", G_CALLBACK (listaller_error_code_cb), plugin);
+	g_signal_connect (upd, "progress", G_CALLBACK (listaller_progress_cb), plugin);
+
+	/* find available updates */
+	listaller_updater_find_updates (upd);
+
+	/* call internal helper function to match selected updates */
+	selected_updates = listaller_get_update_items_from_package_id_list (upd, package_ids);
+
+	ret = listaller_updater_apply_updates (upd, selected_updates);
+
+	g_object_unref (selected_updates);
+	g_object_unref (upd);
+
+	return ret;
+}
+
+/**
+ * pk_plugin_backend_job_package_cb:
  **/
 static void
-pk_plugin_backend_package_cb (PkBackend *backend,
+pk_plugin_backend_job_package_cb (PkBackendJob *job,
 		      PkPackage *package,
 		      PkPlugin *plugin)
 {
@@ -371,10 +513,10 @@ pk_plugin_backend_package_cb (PkBackend *backend,
 }
 
 /**
- * pk_plugin_backend_finished_cb:
+ * pk_plugin_backend_job_finished_cb:
  **/
 static void
-pk_plugin_backend_finished_cb (PkBackend *backend,
+pk_plugin_backend_job_finished_cb (PkBackendJob *job,
 		       PkExitEnum exit_enum,
 		       PkPlugin *plugin)
 {
@@ -382,7 +524,7 @@ pk_plugin_backend_finished_cb (PkBackend *backend,
 		return;
 	if (exit_enum != PK_EXIT_ENUM_SUCCESS) {
 		g_warning ("%s failed with exit code: %s",
-			   pk_role_enum_to_string (pk_backend_get_role (backend)),
+			   pk_role_enum_to_string (pk_backend_job_get_role (job)),
 			   pk_exit_enum_to_string (exit_enum));
 	}
 
@@ -394,10 +536,10 @@ pk_plugin_backend_finished_cb (PkBackend *backend,
 }
 
 /**
- * pk_plugin_backend_error_code_cb:
+ * pk_plugin_backend_job_error_code_cb:
  **/
 static void
-pk_plugin_backend_error_code_cb (PkBackend *backend,
+pk_plugin_backend_job_error_code_cb (PkBackendJob *job,
 				PkError *item,
 				PkPlugin *plugin)
 {
@@ -412,76 +554,71 @@ pk_plugin_backend_error_code_cb (PkBackend *backend,
 
 	/* add to results */
 	pk_results_set_error_code (plugin->priv->backend_results, item);
-	g_debug ("Native backend failed with detail error message: %s", details);
+	g_debug ("Native backend failed with (detailed) error message: %s", details);
 
 	g_free (details);
 }
 
 /**
- * pk_plugin_prepare_backend_call:
- *
- **/
-static PkPluginSignalData*
-pk_plugin_prepare_backend_call (PkPlugin *plugin)
-{
-	PkPluginSignalData *siginfo;
-	PkBitfield	    backend_signals;
-
-	siginfo = g_slice_new (PkPluginSignalData);
-
-	/* connect backend signals to Listaller PkPlugin */
-	siginfo->finished_id = g_signal_connect (plugin->backend, "finished",
-					G_CALLBACK (pk_plugin_backend_finished_cb), plugin);
-	siginfo->package_id = g_signal_connect (plugin->backend, "package",
-				       G_CALLBACK (pk_plugin_backend_package_cb), plugin);
-	siginfo->error_code_id = g_signal_connect (plugin->backend, "error-code",
-				       G_CALLBACK (pk_plugin_backend_error_code_cb), plugin);
-
-	/* don't forward some events to the transaction, only Listaller should see them */
-	backend_signals = PK_TRANSACTION_ALL_BACKEND_SIGNALS;
-	pk_bitfield_remove (backend_signals, PK_BACKEND_SIGNAL_ERROR_CODE);
-	pk_bitfield_remove (backend_signals, PK_BACKEND_SIGNAL_PACKAGE);
-	pk_bitfield_remove (backend_signals, PK_BACKEND_SIGNAL_FINISHED);
-	pk_transaction_set_signals (plugin->priv->current_transaction, backend_signals);
-
-	return siginfo;
-}
-
-/**
- * pk_plugin_restore_backend:
+ * pk_plugin_redirect_backend_signals:
  *
  **/
 static void
-pk_plugin_restore_backend (PkPlugin *plugin, PkPluginSignalData *siginfo)
+pk_plugin_redirect_backend_signals (PkPlugin *plugin)
 {
-	/* disconnect the native backend */
-	if (siginfo->package_id > 0)
-		g_signal_handler_disconnect (plugin->backend, siginfo->package_id);
-	if (siginfo->error_code_id > 0)
-		g_signal_handler_disconnect (plugin->backend, siginfo->error_code_id);
-	if (siginfo->finished_id > 0)
-		g_signal_handler_disconnect (plugin->backend, siginfo->finished_id);
+	if (plugin->priv->error_set == TRUE) {
+		g_debug ("don't redirecting backend signals: Listaller plugin encountered an error.");
+		return;
+	}
 
-	/* connect backend again */
-	pk_transaction_set_signals (plugin->priv->current_transaction, PK_TRANSACTION_ALL_BACKEND_SIGNALS);
-	pk_backend_reset (plugin->backend);
-
-	g_slice_free (PkPluginSignalData, siginfo);
+	/* connect (used) backend signals to Listaller PkPlugin */
+	pk_backend_job_set_vfunc (plugin->job,
+				  PK_BACKEND_SIGNAL_FINISHED,
+				  (PkBackendJobVFunc) pk_plugin_backend_job_finished_cb,
+				  plugin);
+	pk_backend_job_set_vfunc (plugin->job,
+				  PK_BACKEND_SIGNAL_PACKAGE,
+				  (PkBackendJobVFunc) pk_plugin_backend_job_package_cb,
+				  plugin);
+	pk_backend_job_set_vfunc (plugin->job,
+				  PK_BACKEND_SIGNAL_ERROR_CODE,
+				  (PkBackendJobVFunc) pk_plugin_backend_job_error_code_cb,
+				  plugin);
 }
 
 /**
- * pk_backend_request_whatprovides_cb:
+ * pk_plugin_reset_backend_job:
+ *
+ * Reset the PkBackendJob used by the Listaller plugin and rewire
+ * it with the transaction again. (=> does some kind of "soft-reset")
+ **/
+static void
+pk_plugin_reset_backend_job (PkPlugin *plugin)
+{
+	pk_backend_job_reset (plugin->job);
+
+	if (plugin->priv->current_transaction == NULL) {
+		g_critical ("No current transaction is set! Cannot reconnect backend job! (this is a bug)");
+		return;
+	}
+
+	/* connect backend-job to current backend again */
+	pk_transaction_signals_reset (plugin->priv->current_transaction,
+				     plugin->job);
+}
+
+/**
+ * pk_backend_job_request_whatprovides_cb:
  *
  * Helper method for a Listaller native PkBackend proxy to do a what-provides request.
  **/
 static PkResults*
-pk_backend_request_whatprovides_cb (PkBitfield filters,
+pk_backend_job_request_whatprovides_cb (PkBitfield filters,
 				PkProvidesEnum provides,
 				gchar** search,
 				PkPlugin *plugin)
 {
 	PkResults *results;
-	PkPluginSignalData *backend_siginfo;
 
 	/* query the native backend for a package provinding X */
 	if (plugin == NULL)
@@ -491,16 +628,20 @@ pk_backend_request_whatprovides_cb (PkBitfield filters,
 
 	/* prepare for native backend call */
 	pk_plugin_reset (plugin);
-	backend_siginfo = pk_plugin_prepare_backend_call (plugin);
+	pk_plugin_redirect_backend_signals (plugin);
 
 	/* query the native backend */
-	pk_backend_what_provides (plugin->backend, filters, provides, search);
+	pk_backend_what_provides (plugin->backend,
+				   plugin->job,
+				   filters,
+				   provides,
+				   search);
 
 	/* wait for finished */
 	g_main_loop_run (plugin->priv->loop);
 
 	results = plugin->priv->backend_results;
-	pk_plugin_restore_backend (plugin, backend_siginfo);
+	pk_plugin_reset_backend_job (plugin);
 
 	g_debug ("Results exit code is %s", pk_exit_enum_to_string (pk_results_get_exit_code (results)));
 
@@ -508,32 +649,34 @@ pk_backend_request_whatprovides_cb (PkBitfield filters,
 }
 
 /**
- * pk_backend_request_installpackages_cb:
+ * pk_backend_job_request_installpackages_cb:
  *
  * Helper method for a Listaller native PkBackend proxy to do a install-packages request.
  **/
 static PkResults*
-pk_backend_request_installpackages_cb (gboolean only_trusted,
+pk_backend_job_request_installpackages_cb (PkBitfield transaction_flags,
 					gchar **package_ids,
 					PkPlugin *plugin)
 {
 	PkResults *results;
-	PkPluginSignalData *backend_siginfo;
 
 	g_debug ("Running install-packages on native backend!");
 
 	/* prepare the backend */
 	pk_plugin_reset (plugin);
-	backend_siginfo = pk_plugin_prepare_backend_call (plugin);
+	pk_plugin_redirect_backend_signals (plugin);
 
 	/* query the native backend */
-	pk_backend_install_packages (plugin->backend, only_trusted, package_ids);
+	pk_backend_install_packages (plugin->backend,
+				      plugin->job,
+				      transaction_flags,
+				      package_ids);
 
 	/* wait for finished */
 	g_main_loop_run (plugin->priv->loop);
 
 	results = plugin->priv->backend_results;
-	pk_plugin_restore_backend (plugin, backend_siginfo);
+	pk_plugin_reset_backend_job (plugin);
 
 	g_debug ("Results exit code is %s", pk_exit_enum_to_string (pk_results_get_exit_code (results)));
 
@@ -541,7 +684,7 @@ pk_backend_request_installpackages_cb (gboolean only_trusted,
 }
 
 /**
- * pk_plugin_started:
+ * pk_plugin_transaction_started:
  *
  * Hook for the beginning of a new PkTransaction (where it is not completely set-up)
  */
@@ -554,37 +697,79 @@ pk_plugin_transaction_started (PkPlugin *plugin,
 	gchar **package_ids;
 	gchar **data = NULL;
 	gchar **full_paths;
+	gchar *locale;
 
 	ListallerPkBackendProxy *pkbproxy;
 
-	/* reset the native-backend */
-	pk_backend_reset (plugin->backend);
-	pk_backend_set_status (plugin->backend, PK_STATUS_ENUM_SETUP);
-
 	/* set the transaction */
 	plugin->priv->current_transaction = transaction;
 
+	plugin->priv->error_set = FALSE;
+
+	/* reset the native-backend job */
+	pk_plugin_reset_backend_job (plugin);
+	pk_backend_job_set_status (plugin->job, PK_STATUS_ENUM_SETUP);
+
 	/* create a backend proxy and connect it, so Listaller can acces parts of PkBackend */
 	pkbproxy = listaller_pk_backend_proxy_new ();
 	listaller_pk_backend_proxy_set_what_provides (pkbproxy,
-						      (ListallerPkBackendProxyWhatProvidesCB) pk_backend_request_whatprovides_cb,
-						      plugin);
+						(ListallerPkBackendProxyWhatProvidesCB) pk_backend_job_request_whatprovides_cb,
+						plugin);
 	listaller_pk_backend_proxy_set_install_packages (pkbproxy,
-							 (ListallerPkBackendProxyInstallPackagesCB) pk_backend_request_installpackages_cb,
-							 plugin);
+						(ListallerPkBackendProxyInstallPackagesCB) pk_backend_job_request_installpackages_cb,
+						plugin);
 	listaller_set_backend_proxy (pkbproxy);
 
-	/* handle these before the transaction has been run */
+	/* get transaction role */
 	role = pk_transaction_get_role (transaction);
+
+	/* if we're only simulation, skip Listaller packages */
+	if (pk_bitfield_contain (pk_transaction_get_transaction_flags (transaction),
+				   PK_TRANSACTION_FLAG_ENUM_SIMULATE)) {
+
+		if (role == PK_ROLE_ENUM_INSTALL_FILES) {
+			full_paths = pk_transaction_get_full_paths (transaction);
+			data = pk_transaction_filter_listaller_files (transaction,
+									full_paths);
+
+			/* We have Listaller packages, so skip this! */
+			/* FIXME: This needs to be smarter - backend needs to Simulate() with remaining pkgs */
+			if (data != NULL)
+				pk_plugin_skip_native_backend (plugin);
+		} else {
+			/* ignore the return value, we can't sensibly do anything */
+			package_ids = pk_transaction_get_package_ids (transaction);
+			data = pk_transaction_filter_listaller_packages (transaction,
+								package_ids);
+
+			/* nothing more to process */
+			if (data != 0)
+				pk_plugin_skip_native_backend (plugin);
+		}
+
+		goto out;
+	}
+
+	/* set locale so Listaller messages are translated */
+        if (locale = pk_backend_job_get_locale (plugin->job)) {
+            setenv ("LANGUAGE", locale, 1);
+            setenv ("LANG", locale, 1);
+        }
+        g_free (locale);
+
+	g_debug ("Processing transaction");
+
+	/* handle these before the transaction has been run */
 	if (role == PK_ROLE_ENUM_SEARCH_NAME ||
 	    role == PK_ROLE_ENUM_SEARCH_DETAILS) {
 		values = pk_transaction_get_values (transaction);
 		pk_listaller_find_applications (plugin, values);
+
 		goto out;
 	}
 
-	//TODO: PK_ROLE_ENUM_GET_PACKAGES
-	//TODO: PK_ROLE_ENUM_RESOLVE
+	/* TODO: PK_ROLE_ENUM_GET_PACKAGES */
+	/* TODO: PK_ROLE_ENUM_RESOLVE */
 
 	if (role == PK_ROLE_ENUM_GET_DETAILS) {
 		package_ids = pk_transaction_get_package_ids (transaction);
@@ -597,6 +782,7 @@ pk_plugin_transaction_started (PkPlugin *plugin,
 		package_ids = pk_transaction_get_package_ids (transaction);
 		if (g_strv_length (package_ids) == 0)
 			pk_plugin_skip_native_backend (plugin);
+
 		goto out;
 	}
 
@@ -611,33 +797,7 @@ pk_plugin_transaction_started (PkPlugin *plugin,
 		package_ids = pk_transaction_get_package_ids (transaction);
 		if (g_strv_length (package_ids) == 0)
 			pk_plugin_skip_native_backend (plugin);
-		goto out;
-	}
 
-	if (role == PK_ROLE_ENUM_SIMULATE_REMOVE_PACKAGES ||
-	    role == PK_ROLE_ENUM_SIMULATE_INSTALL_PACKAGES) {
-
-		/* ignore the return value, we can't sensibly do anything */
-		package_ids = pk_transaction_get_package_ids (transaction);
-		data = pk_transaction_filter_listaller_packages (transaction,
-							       package_ids);
-
-		/* nothing more to process */
-		package_ids = pk_transaction_get_package_ids (transaction);
-		if (g_strv_length (package_ids) == 0)
-			pk_plugin_skip_native_backend (plugin);
-		goto out;
-	}
-
-	if (role == PK_ROLE_ENUM_SIMULATE_INSTALL_FILES) {
-		full_paths = pk_transaction_get_full_paths (transaction);
-		data = pk_transaction_filter_listaller_files (transaction,
-							    full_paths);
-
-		/* We have Listaller packages, so skip this! */
-		/* FIXME: This needs to be smarter - backend needs to Simulate() with remaining pkgs */
-		if (data != NULL)
-			pk_plugin_skip_native_backend (plugin);
 		goto out;
 	}
 
@@ -654,8 +814,10 @@ pk_plugin_transaction_started (PkPlugin *plugin,
 		full_paths = pk_transaction_get_full_paths (transaction);
 		if (g_strv_length (full_paths) == 0)
 			pk_plugin_skip_native_backend (plugin);
+
 		goto out;
 	}
+
 	if (role == PK_ROLE_ENUM_REMOVE_PACKAGES) {
 		package_ids = pk_transaction_get_package_ids (transaction);
 		data = pk_transaction_filter_listaller_packages (transaction,
@@ -668,6 +830,29 @@ pk_plugin_transaction_started (PkPlugin *plugin,
 		package_ids = pk_transaction_get_package_ids (transaction);
 		if (g_strv_length (package_ids) == 0)
 			pk_plugin_skip_native_backend (plugin);
+
+		goto out;
+	}
+
+	if (role == PK_ROLE_ENUM_GET_UPDATES) {
+		pk_listaller_get_updates (plugin);
+
+		goto out;
+	}
+
+	if (role == PK_ROLE_ENUM_UPDATE_PACKAGES) {
+		package_ids = pk_transaction_get_package_ids (transaction);
+		data = pk_transaction_filter_listaller_packages (transaction,
+							       package_ids);
+
+		if (data != NULL)
+			pk_listaller_apply_updates (plugin, data);
+
+		/* nothing more to process */
+		package_ids = pk_transaction_get_package_ids (transaction);
+		if (g_strv_length (package_ids) == 0)
+			pk_plugin_skip_native_backend (plugin);
+
 		goto out;
 	}
 
@@ -678,25 +863,43 @@ out:
 	listaller_set_backend_proxy (NULL);
 	g_object_unref (pkbproxy);
 	plugin->priv->current_transaction = NULL;
+
+	/* restore locale */
+	setlocale (LC_ALL, "C");
 }
 
 /**
- * pk_plugin_transaction_finished_end:
+ * pk_plugin_transaction_finished_results:
  *
- * Hook for the end of a PkTransaction
+ * Hook for the end (results) of a PkTransaction
  */
 void
-pk_plugin_transaction_finished_end (PkPlugin *plugin,
+pk_plugin_transaction_finished_results (PkPlugin *plugin,
 				    PkTransaction *transaction)
 {
 	PkRoleEnum role;
 
+	if (pk_backend_job_get_exit_code (plugin->job) == PK_EXIT_ENUM_CANCELLED) {
+		g_debug ("skipping finished_results() because transaction was cancelled");
+		return;
+	}
+
+	/* skip simulate actions */
+	if (pk_bitfield_contain (pk_transaction_get_transaction_flags (transaction),
+				 PK_TRANSACTION_FLAG_ENUM_SIMULATE)) {
+		return;
+	}
+
+	/* skip only-download */
+	if (pk_bitfield_contain (pk_transaction_get_transaction_flags (transaction),
+				 PK_TRANSACTION_FLAG_ENUM_ONLY_DOWNLOAD)) {
+		return;
+	}
+
 	/* update application databases */
 	role = pk_transaction_get_role (transaction);
-	if (role == PK_ROLE_ENUM_INSTALL_FILES ||
-	    role == PK_ROLE_ENUM_REMOVE_PACKAGES ||
-	    role == PK_ROLE_ENUM_REFRESH_CACHE) {
-		pk_plugin_listaller_scan_applications (plugin);
+	if (role == PK_ROLE_ENUM_REFRESH_CACHE) {
+		pk_plugin_listaller_refresh_repos (plugin);
 	}
 }
 
@@ -718,23 +921,22 @@ pk_plugin_initialize (PkPlugin *plugin)
 {
 	/* create private area */
 	plugin->priv = PK_TRANSACTION_PLUGIN_GET_PRIVATE (PkPluginPrivate);
-	plugin->priv->conf = listaller_settings_new (TRUE);
-	plugin->priv->mgr = listaller_manager_new (plugin->priv->conf);
+	plugin->priv->mgr = listaller_manager_new (TRUE);
 	plugin->priv->loop = g_main_loop_new (NULL, FALSE);
 	plugin->priv->backend_results = pk_results_new ();
+	plugin->priv->setup_settings = NULL;
 
 	/* tell PK we might be able to handle these */
 	pk_backend_implement (plugin->backend, PK_ROLE_ENUM_GET_DETAILS);
 	pk_backend_implement (plugin->backend, PK_ROLE_ENUM_GET_FILES);
 	pk_backend_implement (plugin->backend, PK_ROLE_ENUM_INSTALL_FILES);
-	pk_backend_implement (plugin->backend, PK_ROLE_ENUM_SIMULATE_INSTALL_FILES);
 	pk_backend_implement (plugin->backend, PK_ROLE_ENUM_REMOVE_PACKAGES);
 
 	/* connect Listaller manager signals */
 	g_signal_connect (plugin->priv->mgr, "error-code", G_CALLBACK (listaller_error_code_cb), plugin);
 	g_signal_connect (plugin->priv->mgr, "message", G_CALLBACK (listaller_message_cb), plugin);
 	g_signal_connect (plugin->priv->mgr, "status-changed", G_CALLBACK (listaller_status_change_cb), plugin);
-	g_signal_connect (plugin->priv->mgr, "progress-changed", G_CALLBACK (listaller_progress_change_cb), plugin);
+	g_signal_connect (plugin->priv->mgr, "progress", G_CALLBACK (listaller_progress_cb), plugin);
 	g_signal_connect (plugin->priv->mgr, "application", G_CALLBACK (listaller_application_cb), plugin);
 
 	/* We want verbose mode! */
@@ -749,7 +951,8 @@ void
 pk_plugin_destroy (PkPlugin *plugin)
 {
 	g_main_loop_unref (plugin->priv->loop);
-	g_object_unref (plugin->priv->conf);
+	if (plugin->priv->setup_settings != NULL)
+		g_object_unref (plugin->priv->setup_settings);
 	g_object_unref (plugin->priv->mgr);
 	g_object_unref (plugin->priv->backend_results);
 }
@@ -762,5 +965,5 @@ pk_plugin_transaction_content_types (PkPlugin *plugin,
 				     PkTransaction *transaction)
 {
 	pk_transaction_add_supported_content_type (transaction,
-						   "application/x-installation");
+						   "application/x-app-package");
 }
